import 'package:rec_api_dart/rec_api_dart.dart';
import 'package:rec_api_dart/src/entities/challenge_reward.ent.dart';

class Challenge extends Entity {
  final ChallengeStatus status;
  final ChallengeAction action;
  final String title;
  final String description;
  final String coverImage;

  final int threshold;
  final int amountRequired;

  final DateTime startDate;
  final DateTime endDate;

  final List<Activity> activities;
  final TokenReward tokenReward;
  final Account owner;

  Challenge({
    String? id,
    String? createdAt,
    String? updatedAt,
    required this.status,
    required this.action,
    required this.title,
    required this.description,
    required this.coverImage,
    required this.threshold,
    required this.amountRequired,
    required this.startDate,
    required this.endDate,
    required this.activities,
    required this.tokenReward,
    required this.owner,
  }) : super(id, createdAt, updatedAt);

  factory Challenge.fromJson(Map<String, dynamic> json) {
    var activities = <Activity>[];

    if (json['activities'] != null) {
      final jsonActivities = List.from(json['activities']);
      activities = jsonActivities.isNotEmpty
          ? jsonActivities.map<Activity>((el) => Activity.fromJson(el)).toList()
          : <Activity>[];
    }

    return Challenge(
      activities: activities,
      coverImage: json['cover_image'],
      description: json['description'],
      endDate: json['end_date'] != null ? DateTime.parse(json['end_date']) : DateTime.now(),
      startDate: json['start_date'] != null ? DateTime.parse(json['start_date']) : DateTime.now(),
      owner: Account.fromJson(json['owner']),
      status: ChallengeStatus.fromName(json['status']),
      action: ChallengeAction.fromName(json['action']),
      threshold: json['threshold'],
      amountRequired: json['amount_required'],
      title: json['title'],
      tokenReward: TokenReward.fromJson(json['token_reward']),
    );
  }
}

enum ChallengeAction {
  BUY,
  QUALIFICATION,
  RECHARGE,
  SEND,
  OTHER;

  String get name {
    switch (this) {
      case ChallengeAction.BUY:
        return 'buy';
      case ChallengeAction.QUALIFICATION:
        return 'qualification';
      case ChallengeAction.RECHARGE:
        return 'recharge';
      case ChallengeAction.SEND:
        return 'send';
      default:
        return 'other';
    }
  }

  static ChallengeAction fromName(String name) {
    switch (name) {
      case 'buy':
        return ChallengeAction.BUY;
      case 'qualification':
        return ChallengeAction.QUALIFICATION;
      case 'recharge':
        return ChallengeAction.RECHARGE;
      case 'send':
        return ChallengeAction.SEND;
      default:
        return ChallengeAction.OTHER;
    }
  }
}

enum ChallengeStatus {
  DRAFT,
  CLOSED,
  OPEN,
  SCHEDULED,
  OTHER;

  get name {
    switch (this) {
      case ChallengeStatus.DRAFT:
        return 'draft';
      case ChallengeStatus.CLOSED:
        return 'closed';
      case ChallengeStatus.OPEN:
        return 'open';
      case ChallengeStatus.SCHEDULED:
        return 'scheduled';
      default:
        return 'other';
    }
  }

  static ChallengeStatus fromName(String name) {
    switch (name) {
      case 'draft':
        return ChallengeStatus.DRAFT;
      case 'closed':
        return ChallengeStatus.CLOSED;
      case 'open':
        return ChallengeStatus.OPEN;
      case 'scheduled':
        return ChallengeStatus.SCHEDULED;
      default:
        return ChallengeStatus.OTHER;
    }
  }
}
