import 'package:http/http.dart';
import 'package:rec_api_dart/src/api_paths.dart';
import 'package:rec_api_dart/src/entities/challenge_reward.ent.dart';
import 'package:rec_api_dart/src/env.base.dart';
import 'package:rec_api_dart/src/interceptors/user_token_interceptor.dart';
import 'package:rec_api_dart/src/interfaces/api_list_response.dart';
import 'package:rec_api_dart/src/services/base_service.dart';

/// This service is in charge of all requests related to [TokenReward]
///
/// * **Authentication Required** - the user must be authenticatedd
class TokenRewardsService extends ServiceBase {
  TokenRewardsService({required EnvBase env, Client? client})
      : super(
          env: env,
          client: client,
          interceptors: [UserTokenInterceptor()],
        );

  /// List all [TokenReward] for the authenticated user
  Future<ApiListResponse<TokenReward?>> list() {
    final uri = ApiPaths.tokenRewards.asUri(env.API_URL);

    return this.get(uri).then(_mapRewards);
  }

  ApiListResponse<TokenReward?> _mapRewards(Map<String, dynamic> resp) {
    return ApiListResponse.fromJson(
      resp['data'],
      mapper: (el) => TokenReward.fromJson(el),
    );
  }
}
